#include "ExceptionalScope.h"
#include "../errorcodes.h"

#include <setjmp.h>
#include <stdlib.h>
#include <string.h>


int  ExceptionalScope_Create(ExceptionalScope* self)
{
	memset(self, 0, sizeof(*self));

	return EXIT_SUCCESS;
}

void ExceptionalScope_ActivateCleaners(ExceptionalScope* self)
{
	while (self->cleaners != NULL) {
		self->cleaners->action(self->cleaners->arg);
		self->cleaners = self->cleaners->next;
	}
}

void ExceptionalScope_Destroy(ExceptionalScope* self)
{
	ExceptionalScope_ActivateCleaners(self);
}

void ExceptionalScope_PushCleaner(ExceptionalScope* self, ExceptionalScopeCleaner* cleaner)
{
	cleaner->next = self->cleaners;
	self->cleaners = cleaner;
}

int ExceptionalScope_PopCleaner(ExceptionalScope* self, ExceptionalScopeCleaner* cleaner)
{
	if (self->cleaners != cleaner) return ENOTFOUND;

	self->cleaners = self->cleaners->next;

	return EXIT_SUCCESS;
}

[[noreturn]] void ExceptionalScope_Throw(ExceptionalScope* self, void* exception)
{
	self->exception = exception;

	ExceptionalScope_ActivateCleaners(self);

	longjmp(self->handler, 1);
}
