#ifndef UTILITIEC_EXCEPTIONALSCOPE_H
#define UTILITIEC_EXCEPTIONALSCOPE_H

#include <setjmp.h>

#define withScopedCleaner(scope, cleaner) for (int __cleanerDone##cleaner = (ExceptionalScope_PushCleaner(scope, &cleaner), 1); __cleanerDone##cleaner--; ExceptionalScope_PopCleaner(scope, &cleaner))
#define try(scope) if (setjmp((scope)->handler) == 0)
#define catch else

typedef void (*ExceptionalScopeCleanerFunction) (void* arg);

typedef struct ExceptionalScopeCleaner_s {
	ExceptionalScopeCleanerFunction action;
	void* arg;

	struct ExceptionalScopeCleaner_s* next;
} ExceptionalScopeCleaner;

typedef struct ExceptionalScope_s {
	jmp_buf handler; // jump here when the exception is thrown
	ExceptionalScopeCleaner* cleaners; // execute all these before the jumping to the handler
	void* exception;
} ExceptionalScope;

int  ExceptionalScope_Create(ExceptionalScope* self);
void ExceptionalScope_Destroy(ExceptionalScope* self);

void ExceptionalScope_PushCleaner(ExceptionalScope* self, ExceptionalScopeCleaner* cleaner);
int  ExceptionalScope_PopCleaner(ExceptionalScope* self, ExceptionalScopeCleaner* cleaner);

[[noreturn]] void ExceptionalScope_Throw(ExceptionalScope* self, void* exception);

#endif
